*========================== Header - Version 2.1 ==========================*
* Program.........:SetDefPr                                                *
* Author..........:Gordon Walker                                           *
* Project.........:General                                                 *
* Description.....:This routine sets the default Windows printer to the    *
*                 :one specified in the tcPrinter parameter. It returns    *
*                 :the name of the previous default printer. This code is  *
*                 :based almost entirely on the sample code provided in    *
*                 :the Microsoft Developer's Knowledge Base article        *
*                 :Q103645 - 'How to Change the Default Windows Printer    *
*                 :from FoxPro'.                                           *
* Post............:The default Windows printer has been changed            *
* Calling Sample..:? SetDefPr( "HP LaserJet 5/5P")                         *
* Parameter List..                                                         *
*      m.tcPrinter  :The name of the printer to be made the default        *
*--------------------------------------------------------------------------*
*                            Major Change List                             *
* DATE: [Wednesday 22nd May, 1996]                                         *
* Added the CurDef() function. This returns the current default printer    *
* name. Used this to add an optimisation. If the current default is the    *
* one specified in tcPrinter then the routine simply returns immediately.  *
* For two consequetive calls for the same printer this improves            *
* performance by an order of magnitude (3.5 sec vs 0.3 sec). If the system *
* is set to print to the default printer then considerable savings should  *
* be achieved.                                                             *
*==========================================================================*
PARAMETERS ;
	m.tcPrinter
PRIVATE ;
	m.lcDefaultPrnt, ;
	laDevice, ;
	m.lnNumPrint, ;
	m.lnPortNum, ;
	m.lcRetVal, ;
	m.lnSpec, ;
	m.lcString

* Load required libraries
IF NOT("FOXTOOLS"$SET("LIBRARY"))
	SET LIBRARY TO FoxTools ADDITIVE
ENDIF

* MOD - JGW - Wednesday 22 May 1996
* Optimise, check to see if the current default is the desired one
IF CurDef()==tcPrinter
	lcRetVal=tcPrinter
ELSE
	* Create an array of printers
	DIMENSION laDevice[1,3]
	lnNumPrint=0
	lcDefaultPrnt=""
	DO GetPrint WITH laDevice, lnNumPrint, lcDefaultPrnt
	
	* Set the return value to indicate the previous
	* default printer
	lcRetVal=LEFT(lcDefaultPrnt,AT(",",lcDefaultPrnt)-1)
	
	* Check that the selected printer has been installed
	IF ASCAN(laDevice,tcPrinter)=0
		=MsgBox("The driver for the "+ALLTRIM(tcPrinter)+ ;
		               " has not been installed.", ;
	       		       "Driver Not Found", ;
	           		   48)
	ELSE
		* Get the matching row number
		lnSpec=ASUBSCRIPT(laDevice,ASCAN(laDevice,tcPrinter),1)
	
		* Assume there is only one port
		lnPortNum = 1 
		lcString=laDevice[lnSpec,1]+","+GetPort(laDevice[lnSpec,2],lnPortNum) 
	  
		* Set the default printer 
		=PutProStrg("windows","device",lcString) 
	ENDIF
ENDIF

RETURN lcRetVal

*!***************************************************************** 
*! 
*!     Procedure: GETPRINT 
*! 
*!***************************************************************** 
PROCEDURE getprint 
* Query WIN.INI to get data on the installed and default printers. 
 
PARAMETER DEVICE, NUMBER, dfltprnt 
* Usage: 
*    DIMENSION device[1] 
*    numprinters = 0 
*    defaultprnt = "" 
*    DO GETPRINT WITH device, numprinters, defaultprnt 
* 
* The "device" array will be populated with the names and 
* parameters for all installed print devices. 
* 
* The "device" array has this structure: 
* Col 1: Printer name 
* Col 2: Parameter for [device] section 
* Col 3: Parameter for [PrinterPort] section (includes time-out 
* parameters) 
* 
* The contents of "device" might look something like this after 
* GETPRINT is called: 
*   Col 1                      Col 2           Col 3 
*   -------------------------  --------------  ---------------- 
*   Apple LaserWriter II NTX   pscript,LPT2:   pscript,LPT2:,15,90 
*   Generic / Text Only=TTY    fred.prn        fred.prn,15,45 
*   HP LaserJets(Level 5)      HPPCL5MS,LPT1:  HPPCL5MS,LPT1:,15,45 
* 
* "numprinters" in this case would be 3 
* "defaultprnt" might be this: 
*    HP LaserJets(Level 5),HPPCL5MS,LPT1: 
  
#DEFINE buflen 2048 
PRIVATE m.in_talk, m.dcount, m.retbuf, m.bytes, m.thisdevice 
  
IF PARAMETERS() < 3 
   WAIT WINDOW "This procedure requires 3 parameters" 
   RETURN 
ENDIF 
  
*IF FILE(SYS(2004)+"FOXTOOLS.FLL") 
*   SET LIBRARY TO (SYS(2004)+"foxtools.fll") ADDITIVE 
*ELSE 
*   WAIT WINDOW "GETPRINT requires the FoxTools library." 
*   RETURN 
*ENDIF 
  
IF SET("TALK") = "ON" 
   SET TALK OFF 
   m.in_talk = "ON" 
ELSE 
   m.in_talk = "OFF" 
ENDIF 
  
* Fill in the first column of the array with installed device 
* names. 
m.retbuf = REPLICATE(CHR(0),buflen) 
m.bytes = getprostrg("devices",0,CHR(0),@m.retbuf,buflen) 
* The second argument of 0 to GetProfileString() returns the contents 
* of the entire section, with each entry separated by a null terminator 
* (CHR(0)). 
m.dcount = 0 
m.retbuf = LEFT(m.retbuf,m.bytes) 
DO WHILE CHR(0) $ m.retbuf 
   m.thisdevice = LEFT(m.retbuf,AT(CHR(0),m.retbuf)-1) 
   IF LEFT(m.thisdevice,1) <> CHR(0) 
      m.dcount = m.dcount + 1 
      DIMENSION device[m.dcount,3] 
      device[m.dcount,1] = m.thisdevice 
   ENDIF 
   m.retbuf = SUBSTR(m.retbuf,AT(CHR(0),m.retbuf)+1) 
ENDDO 
  
* Fill in the second and third columns of the device array with the 
* parameters of each installed device from the [devices] section 
* (column 2) and the [PrinterPorts] section (column 3). 
FOR m.j = 1 TO m.dcount 
   retbuf = REPLICATE(CHR(0),256) 
   m.bytes = ; 
      getprostrg("devices",device[m.j,1],CHR(0),@m.retbuf,256) 
   m.retbuf = LEFT(m.retbuf,m.bytes) 
   device[m.j,2] = m.retbuf 

   retbuf = REPLICATE(CHR(0),256) 
   m.bytes = ; 
      getprostrg("PrinterPorts",device[m.j,1],CHR(0),@m.retbuf,256) 
   m.retbuf = LEFT(m.retbuf,m.bytes) 
   device[m.j,3] = m.retbuf 
ENDFOR 
  
* Store the number of installed devices. 
m.number = m.dcount 
  
* Now get the default printer. 
retbuf = REPLICATE(CHR(0),256) 
m.bytes = getprostrg("windows","device",CHR(0),@m.retbuf,256) 
m.retbuf = LEFT(m.retbuf,m.bytes) 
m.dfltprnt = m.retbuf 
  
SET TALK &in_talk 

*!***************************************************************** 
*! 
*!     Function: GETPROSTRG 
*! 
*!***************************************************************** 
FUNCTION getprostrg 
PARAMETER SECTION, entry, dflt, buffer, blen 
fn = regfn("GETPROFILESTRING","CCC@CI","I") 
RETURN callfn(fn,SECTION,entry,dflt,@buffer,blen) 

*!***************************************************************** 
*! 
*!     Function: GETPORT 
*! 
*!***************************************************************** 
FUNCTION getport 
PARAMETER m.pstrg, m.pnum 
* Get the first "port" from a printer string. 
  
* First get the printer driver name (e.g., pscript) and the comma. 
m.retstrg = SUBSTR(m.pstrg,1,AT(',',m.pstrg)) 
  
* Now get the port designation (e.g., LPT1: or FILE:) 
  
* Check if the port passed is greater than the ports available. 
IF OCCURS(',',m.pstrg) >= m.pnum 
   m.portstrg = SUBSTR(m.pstrg,AT(',',m.pstrg,m.pnum)+1) 
ELSE 
   m.portstrg = SUBSTR(m.pstrg,AT(',',m.pstrg,1)+1) 
ENDIF 
  
IF AT(',',m.portstrg) > 0 
   m.portstrg = LEFT(m.portstrg,AT(',',m.portstrg)-1) 
ENDIF 
  
RETURN m.retstrg + m.portstrg 

*!***************************************************************** 
*! 
*!     Function: PUTPROSTRG 
*! 
*!***************************************************************** 
FUNCTION putprostrg 
PARAMETER SECTION, entry, string 
fn = regfn("WRITEPROFILESTRING","CCC","I") 
RETURN callfn(fn,SECTION,entry,string) 


*-------------------------- Header - Version 2.1 --------------------------*
* Procedure.......:CurDef                                                  *
* Author..........:Gordon Walker                                           *
* Description.....:This routine queries WIN.INI to find the name of the    *
*                 :current default printer                                 *
* Calling Sample..:? CurDef()                                              *
*--------------------------------------------------------------------------*
FUNCTION CurDef
PRIVATE ;
	m.RetBuf, ;
	m.bytes, ;
	m.dfltprnt

* Now get the default printer. 
retbuf = REPLICATE(CHR(0),256) 
m.bytes = getprostrg("windows","device",CHR(0),@m.retbuf,256) 
m.retbuf = LEFT(m.retbuf,m.bytes) 
m.dfltprnt = m.retbuf 

return left(m.dfltprnt,at(',',m.dfltprnt)-1)
